<?php
# vim:ts=2:set noai:
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 sodium                                               |
 | Copyright (C) 2012 Matej Vadnjal                                        |
 | Copyright (C) 2012 Roman Sokolov                                        |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | name    : ss_cisco_sfp_dom.php                                          |
 | version : 0.4.0                                                         |
 | date    : 20121128                                                      |
 +-------------------------------------------------------------------------+
*/

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
  die("<br><strong>This script is only meant to run at the command line.</strong>");
}

$no_http_headers = true;
error_reporting(0);

if (isset($config)) {
  include_once(dirname(__FILE__) . "/../lib/snmp.php");
}

if (!isset($called_by_script_server)) {
  include_once(dirname(__FILE__) . "/../include/global.php");
  include_once(dirname(__FILE__) . "/../lib/snmp.php");
  array_shift($_SERVER["argv"]);
  print call_user_func_array("ss_sfp", $_SERVER["argv"]);
}

# entPhysicalDescr 1.3.6.1.2.1.47.1.1.1.1.2 (.2.[index1|2] = TenGigabitEthernet1/1 Transmit Power Sensor)
# entSensorValue   1.3.6.1.4.1.9.9.91.1.1.1.1.4 (RX = .4.[index1] TX =.4.[index2]) (divide by 10)
# entSensorStatus  1.3.6.1.4.1.9.9.91.1.1.1.1.5 (1 = ok, 2 = unavailable, 3 = nonoperational)
# entSensorType    1.3.6.1.4.1.9.9.91.1.1.1.1.1 (output type 14 = in dBm)
# ifDescr          1.3.6.1.2.1.2.2.1.2 (.2.[ifIndex] = TenGigabitEthernet1/1)
#
# /usr/bin/php -q ss_cisco_sfp_dom.php 192.168.1.11 public 2 index
# /usr/bin/php -q ss_cisco_sfp_dom.php 192.168.1.11 public 2 query index|status|descr
# /usr/bin/php -q ss_cisco_sfp_dom.php 192.168.1.11 public 2 get rx|tx [index]
# 
function ss_sfp($hostname, $snmp_community, $snmp_version, $cmd, $want_key = "", $index = "") {

  $snmp = array();
  $snmp['retries'] = read_config_option("snmp_retries");
  $snmp['hostname'] = $hostname;
  $snmp['community'] = $snmp_community;
  $snmp['version'] = $snmp_version;
  $snmp['port'] = 161;
  $snmp['timeout'] = 1000;
  $snmp['username'] = '';
  $snmp['password'] = '';
  $snmp['auth_proto'] = '';
  $snmp['priv_pass'] = '';
  $snmp['priv_proto'] = '';
  $snmp['context'] = '';
  $snmp['max_oids'] = 10;
  $snmp_retries = read_config_option("snmp_retries");

  $interface = '';
  $sensors = array();
  $query_rows = array();

  if ($cmd == "index" || $cmd == "query") {
    $var_sensor = ss_sfp_walk(".1.3.6.1.4.1.9.9.91.1.1.1.1.1", $snmp);
    foreach ($var_sensor as $i => $row) {
      if (preg_match('/\.(\d+)$/', $row['oid'], $matches)) {
        $sensors[$matches[1]]['type'] = $row['value'];
      }
    }
    if (sizeof($sensors) < 1) {
      return;
    }
    $var_sensor = ss_sfp_walk(".1.3.6.1.4.1.9.9.91.1.1.1.1.4", $snmp);
    foreach ($var_sensor as $i => $row) {
      if (preg_match('/\.(\d+)$/', $row['oid'], $matches)) {
        $sensors[$matches[1]]['value'] = $row['value'];
      }
    }
    $var_sensor = ss_sfp_walk(".1.3.6.1.4.1.9.9.91.1.1.1.1.5", $snmp);
    foreach ($var_sensor as $i => $row) {
      if (preg_match('/\.(\d+)$/', $row['oid'], $matches)) {
        $sensors[$matches[1]]['status'] = $row['value'];
      }
    }

    $var_devices = ss_sfp_walk(".1.3.6.1.2.1.47.1.1.1.1.2", $snmp);
    foreach ($var_devices as $i => $row) {
      if (preg_match('/\.(\d+)$/', $row['oid'], $matches)) {
        $devices[$matches[1]]['name'] = $row['value'];
      }
    }
    if (sizeof($devices) < 1) {
      return;
    }
    $var_devices = ss_sfp_walk(".1.3.6.1.2.1.47.1.1.1.1.4", $snmp);
    foreach ($var_devices as $i => $row) {
      if (preg_match('/\.(\d+)$/', $row['oid'], $matches)) {
        $devices[$matches[1]]['parent'] = $row['value'];
      }
    }
    $var_devices = ss_sfp_walk(".1.3.6.1.2.1.47.1.1.1.1.7", $snmp);
    foreach ($var_devices as $i => $row) {
      if (preg_match('/\.(\d+)$/', $row['oid'], $matches)) {
        $devices[$matches[1]]['truename'] = $row['value'];
      }
    }

    foreach ($sensors as $sensor_index => $sensor) {

      if (!(($sensor['type'] == "6") || ($sensor['type'] == "14"))) {
        // not a Power Sensor
        continue;
      }

      $var_parent=$sensor_index;
      while ( (!preg_match('/^(Transceiver Port)/', $devices[$var_parent]['name'], $matches)) &&
              (!preg_match('/(Port)$/', $devices[$var_parent]['name'], $matches)) &&
              ($var_parent>0)) {
        $var_parent=$devices[$var_parent]['parent'];
      }
      if ($var_parent!=0) {
        $sensor['name'] = $devices[$var_parent]['truename'];
      }
      else {
        continue;
      }

      $interface = $sensor['name'];

      if (preg_match('/^(.+) R.* Power Sensor$/', $devices[$sensor_index]['name'], $matches)) {
        $direction = 'rx';
      }
      elseif (preg_match('/^(.+) T.* Power Sensor$/', $devices[$sensor_index]['name'], $matches)) {
        $direction = 'tx';
      }
      else {
        continue;
      }
      foreach ($sensor as $sensor_key => $sensor_value) {
        $query_rows[$interface][$direction.'_'.$sensor_key] = $sensor_value;
      }
      $query_rows[$interface][$direction.'_index'] = $sensor_index;
      $query_rows[$interface][$direction.'_type'] = $sensor['type'];
      $query_rows[$interface]['interface'] = $interface;
    }

    foreach ($query_rows as $interface => &$query_row) {
      ss_sfp_status_string($query_row);
      ss_sfp_query_index($query_row);
      ss_sfp_iface_details($query_row, $snmp);
      $query_row['tx_value'] = ss_sfp_fix_value($query_row['tx_value'],$query_row['tx_type']);
      $query_row['rx_value'] = ss_sfp_fix_value($query_row['rx_value'],$query_row['rx_type']);

      if ($cmd == 'index') {
        print $query_row['index']."\n";
      }
      elseif ($cmd == 'query') {
        switch ($want_key) {
          case "index":
            print $query_row['index'].":".$query_row['index']."\n";
          break;
          case "status":
            print $query_row['index'].":".$query_row['status']."\n";
          break;
          case "interface":
            print $query_row['index'].":".$query_row['interface']."\n";
          break;
          case "if_name":
            print $query_row['index'].":".$query_row['if_name']."\n";
          break;
          case "descr":
            print $query_row['index'].":".$query_row['descr']."\n";
          break;
          case "rx":
            print $query_row['index'].":".$query_row['rx_value']."\n";
          break;
          case "tx":
            print $query_row['index'].":".$query_row['tx_value']."\n";
          break;
        }
      }
    }
  }
  elseif ($cmd == "get") {
    $idxs = explode('.', $index);
    if ($want_key == 'tx') {
      $var_idxs = $idxs[0];
    } elseif ($want_key == 'rx') {
      $var_idxs = $idxs[1];
    } else {
      return 'NaN';
    }
    $sensor_status = ss_sfp_get('.1.3.6.1.4.1.9.9.91.1.1.1.1.5.'.$var_idxs, $snmp);
    if ($sensor_status != 1)
      return ss_sfp_fix_value(-400,14);
    $var_value = ss_sfp_get('.1.3.6.1.4.1.9.9.91.1.1.1.1.4.'.$var_idxs, $snmp);
    $var_type = ss_sfp_get('.1.3.6.1.4.1.9.9.91.1.1.1.1.1.'.$var_idxs, $snmp);
    return ss_sfp_fix_value($var_value,$var_type);
  }
  elseif ($cmd == "num_indexes") {
    $var = ss_sfp_walk(".1.3.6.1.4.1.9.9.91.1.1.1.1.1", $snmp);
    return sizeof($var);
  }
}

function ss_sfp_status_string(&$row) {
  if ($row['tx_status'] == 1 and $row['rx_status'] == 1) {
    $row['status'] = 'Online';
  }
  elseif ($row['tx_status'] == 1 and $row['rx_status'] != 1) {
    $row['status'] = 'Rx Failure';
    $row['rx_value'] = -400;
  }
  elseif ($row['tx_status'] != 1 and $row['rx_status'] == 1) {
    $row['status'] = 'Tx Failure';
    $row['tx_value'] = -400;
  }
  elseif ($row['tx_status'] != 1 and $row['rx_status'] != 1) {
    $row['status'] = 'Tx and Rx Failure';
    $row['tx_value'] = -400;
    $row['rx_value'] = -400;
  }
}

function ss_sfp_iface_details(&$row, $snmp) {
  /* getting if_index from interface name */
  $if_index = -1;
  $row['descr'] = '';
  $row['if_name'] = '';

  $var_ifdescrs = ss_sfp_walk(".1.3.6.1.2.1.2.2.1.2", $snmp);
  foreach ($var_ifdescrs as $if_row) {
    if ($row['interface'] == $if_row['value']) {
      $if_index = preg_replace('/.*[\.0-9]*\.([0-9]+)$/', '\\1', $if_row['oid']);
      $row['descr'] = ss_sfp_get('.1.3.6.1.2.1.31.1.1.1.18.'.$if_index, $snmp);
      $row['if_name'] = ss_sfp_get('.1.3.6.1.2.1.31.1.1.1.1.'.$if_index, $snmp);
      return;
    }
  }
  $var_ifnames = ss_sfp_walk(".1.3.6.1.2.1.31.1.1.1.1", $snmp);
  foreach ($var_ifnames as $if_row) {
    if ($row['interface'] == $if_row['value']) {
      $if_index = preg_replace('/.*[\.0-9]*\.([0-9]+)$/', '\\1', $if_row['oid']);
      $row['descr'] = ss_sfp_get('.1.3.6.1.2.1.31.1.1.1.18.'.$if_index, $snmp);
      $row['if_name'] = $row['interface'];
      $row['name'] = ss_sfp_get('.1.3.6.1.2.1.2.2.1.2.'.$if_index, $snmp);
      return;
    }
  }
}

function ss_sfp_query_index(&$row) {
  $row['index'] = sprintf('%s.%s', $row['tx_index'], $row['rx_index']);
}

function ss_sfp_fix_value($val,$type) {
  if ($type == "6") {
    $val = 10*log($val/1000,10);
  } else {
    $val = $val/10;
  }
  if ($val === '' or $val === false or $val < -39.9) {
    $val = -40; // lights are off
  }
  return floatval($val);
}

function ss_sfp_walk($oid, $snmp) {
  return cacti_snmp_walk(
    $snmp['hostname'],
    $snmp['community'],
    $oid,
    $snmp['version'],
    $snmp['username'],
    $snmp['password'],
    $snmp['auth_proto'],
    $snmp['priv_pass'],
    $snmp['priv_proto'],
    $snmp['context'],
    $snmp['port'],
    $snmp['timeout'],
    $snmp['retries'],
    $snmp['max_oids'],
    SNMP_POLLER
  );
}

function ss_sfp_get($oid, $snmp) {
  return cacti_snmp_get(
    $snmp['hostname'],
    $snmp['community'],
    $oid,
    $snmp['version'],
    $snmp['username'],
    $snmp['password'],
    $snmp['auth_proto'],
    $snmp['priv_pass'],
    $snmp['priv_proto'],
    $snmp['context'],
    $snmp['port'],
    $snmp['timeout'],
    $snmp['retries'],
    SNMP_POLLER
  );
}

?>
